@@ -5,9 +5,8 @@ mod records {
 	#[test]
 	fn fields() {
 		let cache = new_cache!().unwrap();
-		let pkg = cache.get("apt").unwrap();
-		// let pkg = cache.get("nala").unwrap();
-		let cand = pkg.candidate().unwrap();
+
+		let cand = cache.get("apt").unwrap().candidate().unwrap();
 
 		assert_eq!(
 			cand.get_record(RecordField::Maintainer).unwrap(),
@@ -16,9 +15,6 @@ mod records {
 		// Apt should not have a homepage
 		assert!(cand.get_record(RecordField::Homepage).is_none());
 
-		// The apt source field be none as it is just "apt"
-		assert!(cand.get_record(RecordField::Source).is_some());
-
 		// This should also equal the same as the cand version
 		assert_eq!(
 			cand.get_record(RecordField::Version).unwrap(),
